<?xml version="1.0" encoding="US-ASCII"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:c="http://www.CraneSoftwrights.com/ns/ublsec"
                version="1.0" exclude-result-prefixes="c">

<!--

Add the required signature scaffolding to a UBL document.  The 'xmlsec'
utility expects to find a template into which it injects the signature
information.

$Id: Crane-DetachedUBL.xsl,v 1.3 2010/11/27 02:50:03 admin Exp $

Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-dev.htm

Redistribution and use of components created by Crane Softwrights Ltd. in 
source and binary forms, with or without modification, are permitted 
provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- The name of the author may not be used to endorse or promote products
  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5

THE AUTHOR MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS CODE FOR ANY
PURPOSE. PLEASE CONSULT THE RESPECTIVE RESPONSIBLE PARTIES REGARDING COPYRIGHT
AND LEGAL OBLIGATIONS FOR COMPONENTS NOT WRITTEN BY CRANE SOFTWRIGHTS LTD.
-->

<xsl:import href="Crane-SigObject.xsl"/>

<!--UBL optionally added information-->
<xsl:param name="final"/>
<xsl:param name="uri"/>
<xsl:param name="SigID"/>
<xsl:param name="SigValID"/>
<xsl:param name="digestMethod"
           select="'http://www.w3.org/2000/09/xmldsig#sha1'"/>

<!--the document element that doesn't have any extensions, so add sig one-->
<xsl:template match="/" name="c:Signature">
  <xsl:param name="calledByEnveloped" select="false()"/>
  <xsl:variable name="use-final">
    <xsl:choose xmlns:ext=
    "urn:oasis:names:specification:ubl:schema:xsd:CommonExtensionComponents-2"
                xmlns:sig=
    "urn:oasis:names:specification:ubl:schema:xsd:CommonSignatureComponents-2">
      <xsl:when test="$final!='final' and not($calledByEnveloped) and
                      not(/*/ext:UBLExtensions/ext:UBLExtension/
                          ext:ExtensionContent/sig:UBLDocumentSignatures)">
        <xsl:message>
          <xsl:text>WARNING: the "final" argument is assumed to be </xsl:text>
          <xsl:text>"final" (the original value of "</xsl:text>
          <xsl:value-of select="$final"/>
          <xsl:text>" is ignored) because </xsl:text>
          <xsl:text>there are no pre-existing UBL signatures to </xsl:text>
          <xsl:text>protect in the document at "</xsl:text>
          <xsl:value-of select="$uri"/>
          <xsl:text>".
</xsl:text>
        </xsl:message>
        <xsl:text>final</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$final"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <!--determine if there is a transformation for the digest calculation-->
  <xsl:variable name="XPath">
    <xsl:choose>
      <xsl:when test="not($calledByEnveloped) 
                      and $use-final!='final'"><!--not final; detached-->
        count(ancestor-or-self::sig:UBLDocumentSignatures)=0
          </xsl:when>
      <xsl:when test="not($calledByEnveloped)"/><!--final; detached-->
      <xsl:when test="$final='final'"><!--final; enveloped-->
            count(ancestor-or-self::ds:Signature |
                   here()/ancestor::ds:Signature[1]) >
            count(ancestor-or-self::ds:Signature)
          </xsl:when>
      <xsl:otherwise><!--not final; enveloped-->
            count(ancestor-or-self::sig:UBLDocumentSignatures |
                  here()/ancestor::sig:UBLDocumentSignatures[1]) >
            count(ancestor-or-self::sig:UBLDocumentSignatures)
          </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
<xsl:if test="true()" xml:space="preserve"><!--use this for indentation-->
<ds:Signature xmlns:ds="http://www.w3.org/2000/09/xmldsig#"><xsl:if 
  test="$SigID" xml:space="default">
    <xsl:attribute name="Id"><xsl:value-of select="$SigID"/></xsl:attribute>
  </xsl:if>
  <ds:SignedInfo>
    <ds:CanonicalizationMethod 
                  Algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315"/>
    <ds:SignatureMethod 
                       Algorithm="http://www.w3.org/2000/09/xmldsig#rsa-sha1"/>
    <ds:Reference URI="{$uri}"><xsl:if test="string($XPath)">
      <ds:Transforms>
        <ds:Transform Algorithm="http://www.w3.org/TR/1999/REC-xpath-19991116">
          <ds:XPath xmlns:sig=
    "urn:oasis:names:specification:ubl:schema:xsd:CommonSignatureComponents-2"
            ><xsl:value-of select="$XPath"/></ds:XPath>
        </ds:Transform>
      </ds:Transforms></xsl:if>
      <ds:DigestMethod Algorithm="{$digestMethod}"/>
      <ds:DigestValue/>
    </ds:Reference><xsl:call-template name="c:SigReferences"/>
  </ds:SignedInfo>
  <ds:SignatureValue><xsl:if test="$SigValID" xml:space="default">
    <xsl:attribute name="Id">
      <xsl:value-of select="$SigValID"/>
    </xsl:attribute>
  </xsl:if></ds:SignatureValue>
  <ds:KeyInfo>
    <ds:KeyValue/>
    <ds:X509Data>
      <ds:X509SubjectName/>
      <ds:X509IssuerSerial/>
      <ds:X509Certificate/>
    </ds:X509Data>
  </ds:KeyInfo>
<xsl:call-template name="c:SigObject"/></ds:Signature>
</xsl:if>
</xsl:template>

</xsl:stylesheet>