<?xml version="1.0" encoding="US-ASCII"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:c="http://www.CraneSoftwrights.com/ns/ublsec"
xmlns:cbc="urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2"
xmlns:ext="urn:oasis:names:specification:ubl:schema:xsd:CommonExtensionComponents-2"
xmlns:sig="urn:oasis:names:specification:ubl:schema:xsd:CommonSignatureComponents-2" 
xmlns:sac="urn:oasis:names:specification:ubl:schema:xsd:SignatureAggregateComponents-2" 
xmlns:sbc="urn:oasis:names:specification:ubl:schema:xsd:SignatureBasicComponents-2"
version="1.0" exclude-result-prefixes="c">

<!--

Add the required signature scaffolding to a UBL document.  The 'xmlsec'
utility expects to find a template into which it injects the signature
information.

$Id: Crane-EnvelopedUBL.xsl,v 1.4 2010/11/27 02:50:04 admin Exp $

Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-dev.htm

Redistribution and use of components created by Crane Softwrights Ltd. in 
source and binary forms, with or without modification, are permitted 
provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- The name of the author may not be used to endorse or promote products
  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5

THE AUTHOR MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS CODE FOR ANY
PURPOSE. PLEASE CONSULT THE RESPECTIVE RESPONSIBLE PARTIES REGARDING COPYRIGHT
AND LEGAL OBLIGATIONS FOR COMPONENTS NOT WRITTEN BY CRANE SOFTWRIGHTS LTD.
-->

<xsl:import href="Crane-DetachedUBL.xsl"/>

<!--UBL optionally added information-->
<xsl:param name="UBLID"/>
<xsl:param name="UBLReferencedSignatureID"/>

<!--this is needed because the detached stylesheet is matching on this node-->
<xsl:template match="/"><xsl:apply-templates/></xsl:template>

<!--the document element that doesn't have any extensions, so add sig one-->
<xsl:template match="/*[not(ext:UBLExtensions)]">
  <xsl:copy>
    <xsl:copy-of select="@*"/>
    <!--add the signatures as the only extensions to the document-->
    <ext:UBLExtensions>
      <xsl:call-template name="c:AddSignatureBlock"/>
    </ext:UBLExtensions>
    <xsl:copy-of select="node()"/>
  </xsl:copy>
</xsl:template>

<!--there are existing extensions but no sig, so add sig at the end-->
<xsl:template match="ext:UBLExtensions[not(ext:UBLExtension/
                     ext:ExtensionContent/sig:UBLDocumentSignatures)]">
  <xsl:copy>
    <xsl:copy-of select="@*"/>
    <xsl:copy-of select="node()"/>
    <!--add the signature as additional extension information-->
    <xsl:call-template name="c:AddSignatureBlock"/>
  </xsl:copy>
</xsl:template>

<!--there is already a signature block, add the new one at the end-->
<xsl:template match="sig:UBLDocumentSignatures">
  <xsl:copy>
    <xsl:copy-of select="@*"/>
    <xsl:copy-of select="node()"/>
    <xsl:call-template name="c:AddSignature"/>
  </xsl:copy>
</xsl:template>

<!--a block of signatures must be added-->
<xsl:template name="c:AddSignatureBlock" xml:space="preserve"
  ><ext:UBLExtension>
    <ext:ExtensionContent>
      <sig:UBLDocumentSignatures><xsl:call-template name="c:AddSignature"/>
      </sig:UBLDocumentSignatures>
    </ext:ExtensionContent>
  </ext:UBLExtension>
</xsl:template>

<!--add a single signature-->
<xsl:template name="c:AddSignature">
  <xsl:text>
</xsl:text>
  <sac:SignatureInformation>
    <!--some metadata might be in the invocation parameters-->
    <xsl:if test="$UBLID">
      <xsl:text>
</xsl:text>
      <cbc:ID><xsl:value-of select="$UBLID"/></cbc:ID>
    </xsl:if>
    <xsl:if test="$UBLReferencedSignatureID">
      <xsl:text>
</xsl:text>
      <sbc:ReferencedSignatureID>
        <xsl:value-of select="$UBLReferencedSignatureID"/>
      </sbc:ReferencedSignatureID>
    </xsl:if>
    <!--build up the template to be filled in by the xmlsec utility-->
    <xsl:call-template name="c:Signature">
      <xsl:with-param name="calledByEnveloped" select="true()"/>
    </xsl:call-template>
  </sac:SignatureInformation>
</xsl:template>

<!--identity for all other nodes-->
<xsl:template match="@*|node()">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()"/>
  </xsl:copy>
</xsl:template>

</xsl:stylesheet>