@echo off
goto :start

rem Add an enveloped digital signature to a UBL document using a PKCS12 file.
rem
rem Use five to eight arguments (the last three are optional, use "" to skip):
rem     inputDoc outputDoc final pkcs12File privateKey 
rem                        [ SigID [ UBLID [ UBLReferencedID ] ] ]
rem 
rem $Id: envelopedUBL.bat,v 1.4 2010/11/27 21:40:07 admin Exp $
rem
rem Copyright (C) - Crane Softwrights Ltd.
rem               - http://www.CraneSoftwrights.com/links/res-dev.htm
rem 
rem Redistribution and use of components created by Crane Softwrights Ltd. in 
rem source and binary forms, with or without modification, are permitted 
rem provided that the following conditions are met:
rem 
rem - Redistributions of source code must retain the above copyright notice,
rem   this list of conditions and the following disclaimer.
rem - Redistributions in binary form must reproduce the above copyright notice,
rem   this list of conditions and the following disclaimer in the documentation
rem   and/or other materials provided with the distribution.
rem - The name of the author may not be used to endorse or promote products
rem   derived from this software without specific prior written permission.
rem 
rem THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
rem IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
rem OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
rem NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
rem SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
rem TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
rem PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
rem LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
rem NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE
rem EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
rem 
rem Note:for your reference, the above is the "Modified BSD license", this text
rem   was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
rem 
rem THE AUTHOR MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS CODE FOR 
rem ANY PURPOSE. PLEASE CONSULT THE RESPECTIVE RESPONSIBLE PARTIES REGARDING 
rem COPYRIGHT AND LEGAL OBLIGATIONS FOR COMPONENTS NOT WRITTEN BY CRANE 
rem SOFTWRIGHTS LTD.

:start
set signPath=%~dp0
shift
set signIn=%~0
shift
set signOut=%~0
shift
if "a%signIn%" == "a" goto :help
if not "a%~9" == "a" goto :help
set signUBL=0

if not exist "%signIn%" goto :notfound
if exist "%signOut%" del "%signOut%"
if exist "%signOut%" goto :nodelete
if "a%~0" == "afinal" goto :final-notfinal
if "a%~0" == "anotfinal" goto :final-notfinal
goto :badfinal
:final-notfinal
if not exist "%~1" goto :nokey
if "a%~2" == "a" goto :nopass
set signSigID=
if not "a%~3" == "a" set signSigID="SigID=%~3"
set signSigValID=
if not "a%~4" == "a" set signSigValID="SigValID=%~4"
set signUBLID=
if not "a%~5" == "a" set signUBLID="UBLID=%~5"
set signUBLReferencedSignatureID=
if not "a%~6" == "a" set signUBLReferencedSignatureID="UBLReferencedSignatureID=%~6"
set signObj=
if not "a%~7" == "a" set signObj="objectFileName=%~7"
set signRefs=
if not "a%~8" == "a" set signRefs="RefIDs=%~8"

set signIDAttrs=--id-attr:Id "http://www.w3.org/2000/09/xmldsig#:Signature" --id-attr:Id "http://www.w3.org/2000/09/xmldsig#:SignatureValue"

if not exist "%signPath%saxon.jar" goto :nosaxon
echo Adding UBL signature scaffolding at the end of the signatures...
java -jar "%signPath%saxon.jar" -o "%signOut%.raw" "%signIn%" "%signPath%Crane-EnvelopedUBL.xsl" %signUBLID% %signUBLReferencedSignatureID% %signSigID% %signSigValID% %signObj% %signRefs% "final=%~0"
set signUBL=%errorlevel%
set signSigID=
set signUBLID=
set signUBLReferencedSignatureID=
if %signUBL% neq 0 goto :badaug
echo Injecting signature information into the last scaffolding found...
xmlsec --sign --node-xpath "(//*[local-name()='Signature' and namespace-uri()='http://www.w3.org/2000/09/xmldsig#'])[last()]" %signIDAttrs% --pkcs12 "%~1" --pwd %2 "%signOut%.raw" > "%signOut%"

set signUBL=%errorlevel%
del "%signOut%.raw"
if %signUBL% neq 0 goto :badsign
"%signPath%verifyUBL" "%signOut%"
rem It shouldn't get here because of the above is not called, but jump anyway
goto :done

:nosaxon
echo Cannot find "%signPath%saxon.jar"
goto :error
:notfound
echo Input file "%signIn%" not found
goto :error
:nodelete
echo Output file "%signOut%" cannot be overwritten
goto :error
:badfinal
echo Third argument must be either "final" or "notfinal": "%~0"
goto :error
:nokey
echo PKCS#12 file "%~1" not found
goto :error
:nopass
echo Private key not specified
goto :error
:badaug
echo Stylesheet augmenting signature information failed
if exist "%signOut%" del "%signOut%"
goto :error
:badsign
echo Signing software returned an error
goto :error
:error
echo Execution terminated.
:help
echo.
echo Add an enveloped digital signature to a UBL document using a PKCS12 file.
echo.
echo Use five to ten arguments (the last four are optional, use "" to skip):
echo     inputDoc outputDoc final pkcs12File privateKey 
echo         [ SigID [ SigValID [ UBLID [ UBLReferencedID [ objectDoc ] ] ] ] ]
set signUBL=1
:done
exit /b %signUBL%
